import os
import configparser
import semester_planner.config as cfg
from typing import Any


class SemesterPlannerConfig:
    def __init__(self):
        self.config_parser = configparser.ConfigParser()
        self.config_parser.read(cfg.CONFIG_FILE_PATH)

    def save_default_config(self):
        for section in cfg.DEFAULT_LOCAL_CONFIG:
            if section not in self.config_parser:
                self.config_parser[section] = {}
            for key, value in cfg.DEFAULT_LOCAL_CONFIG[section].items():
                if key not in self.config_parser[section]:
                    self.config_parser[section][key] = str(value)

    def __getitem__(self, section: str):
        if section not in self.config_parser:
            self.config_parser[section] = {}
        return self.config_parser[section]

    def __setitem__(self, section: str, value: dict):
        self.config_parser[section] = value

    def save(self):
        self.config_parser.write(open(cfg.CONFIG_FILE_PATH, 'w+'))

    def to_string(self, section: str):
        if section not in self.config_parser:
            return "Empty section."
        result = "[{}]\n".format(section)
        for key, value in self.config_parser[section].items():
            result += '{} = {}\n'.format(key, value)
        return result

    def __str__(self):
        result = 'Semester planner configuration. ({})\n'.format(
            cfg.CONFIG_FILE_PATH)
        result += '\n'
        for section in self.config_parser:
            result += self.to_string(section)



import argparse
import os


def _process_args(args: argparse.Namespace):
    if args.semester is None and args.command != 'config':
        for file in os.listdir(os.getcwd()):
            if os.path.isfile(file):
                file_name, ext = os.path.splitext(file)
                if 'semester' in file_name and ext == '.json':
                    args.semester = os.path.join(os.getcwd(), file)
                    break
        else:
            raise FileNotFoundError("Cannot find semester configuration file")


def _setup_todoist_argparse(subparsers: argparse._SubParsersAction):
    todoist_parser = subparsers.add_parser('todoist')
    todoist_parser.add_argument(
        '-t', '--token',
        help='Todoist token.'
    )
    todoist_parser.add_argument(
        '-r', '--root-project',
        help='root project to setup.',
        required=False
    )
    todoist_parser.add_argument(
        '-le', '--lectures',
        help='setup lectures at Todoist.',
        action='store_true'
    )
    todoist_parser.add_argument(
        '-la', '--labs',
        help='setup labs at Todoist.',
        action='store_true'
    )
    todoist_parser.add_argument(
        '-pr', '--practical',
        help='setup practical at Todoist.',
        action='store_true'
    )
    todoist_parser.add_argument(
        '-a', '--all',
        help='setup all at Todoist.',
        action='store_true'
    )
    todoist_parser.add_argument(
        '-o', '--override',
        help='override existing projects.',
        action='store_true'
    )


def _setup_config_argparse(subparsers: argparse._SubParsersAction):
    config_parser = subparsers.add_parser('config')

    config_subparsers = config_parser.add_subparsers(dest='config_section')

    config_todoist_parser = config_subparsers.add_parser('todoist')
    config_todoist_parser.add_argument(
        '-t', '--token',
        help='Todoist token.',
        required=False
    )
    config_todoist_parser.add_argument(
        '-r', '--root',
        help='root project to setup.',
        required=False
    )

    config_parser.add_argument(
        '-l', '--list',
        help='list configurations.',
        action='store_true'
    )


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-s', '--semester',
        help='file with semester configurations',
        required=False
    )

    subparsers = parser.add_subparsers(dest='command')
    _setup_todoist_argparse(subparsers)
    _setup_config_argparse(subparsers)

    args = parser.parse_args()
    _process_args(args)
    return args



"""Collection of dataholders."""
import datetime
from typing import Union, List


class Class:
    def __init__(self, subject: str, date: datetime.date, number: int):
        self.subject = subject
        self.date = date
        self.number = number


class ClassIterator:
    def __init__(self, class_data):
        self.class_data = class_data
        self.current_date = class_data.start
        self.current_number = 0
        while (self.current_date.weekday() in self.class_data.dayoffs):
            self.current_date += self.class_data.interval

    def __next__(self):
        if self.current_date >= self.class_data.end:
            raise StopIteration
        obj = Class(
            subject=self.class_data.subject,
            date=self.current_date,
            number=self.current_number
        )
        self.current_date += self.class_data.interval
        while (self.current_date.weekday() in self.class_data.dayoffs):
            self.current_date += self.class_data.interval
        self.current_number += 1
        return obj


class ClassData:
    DATE_FORMAT = '%m/%d/%Y'

    def __init__(self,
                 subject: str,
                 start: Union[str, datetime.date],
                 end: Union[str, datetime.date],
                 interval: Union[int, datetime.timedelta],
                 dayoffs: List[Union[str, int]]):

        self.subject = subject
        if isinstance(start, datetime.date):
            self.start = start
        else:
            self.start = datetime.datetime.strptime(
                start, ClassData.DATE_FORMAT).date()

        if isinstance(end, datetime.date):
            self.end = end
        else:
            self.end = datetime.datetime.strptime(
                end, ClassData.DATE_FORMAT).date()

        if isinstance(interval, datetime.timedelta):
            self.interval = interval
        else:
            self.interval = datetime.timedelta(interval)

        self.dayoffs = []
        for dayoff in dayoffs:
            dayoff = dayoff.lower()
            if dayoff == 'mon' or dayoff == 'monday' or dayoff == 0:
                self.dayoffs.append(0)
            elif dayoff == 'tue' or dayoff == 'tues' or dayoff == 'tuesday' or dayoff == 1:
                self.dayoffs.append(1)
            elif dayoff == 'wed' or dayoff == 'wednesday' or dayoff == 2:
                self.dayoffs.append(2)
            elif dayoff == 'thu' or dayoff == 'thurs' or dayoff == 'thursday' or dayoff == 3:
                self.dayoffs.append(3)
            elif dayoff == 'fri' or dayoff == 'friday' or dayoff == 4:
                self.dayoffs.append(4)
            elif dayoff == 'sat' or dayoff == 'saturday' or dayoff == 5:
                self.dayoffs.append(5)
            elif dayoff == 'sun' or dayoff == 'sunday' or dayoff == 6:
                self.dayoffs.append(6)

    def __iter__(self):
        return ClassIterator(self)

    @staticmethod
    def parse(
        obj: dict,
        **defaults
    ):
        """Build ClassData object from dict with provided defaults

        Arguments:
            obj {dict} -- dict object with the data about the class
        """
        if 'start' in obj:
            start = obj['start']
        else:
            start = defaults['start']

        if 'interval' in obj:
            interval = obj['interval']
        else:
            interval = defaults.get('interval', 1)

        if 'end' in obj:
            end = obj['end']
        elif 'count' in obj:
            count = obj['count']
            end = datetime.datetime.strptime(
                start,
                ClassData.DATE_FORMAT
            ).date() + datetime.timedelta(interval * count)
        else:
            end = defaults['end']

        if 'dayoffs' in obj:
            dayoffs = obj['dayoffs']
        else:
            dayoffs = defaults.get('dayoffs', [])

        if 'subject' in obj:
            subject = obj['subject']
        else:
            subject = defaults['subject']

        return ClassData(
            subject=subject,
            start=start,
            end=end,
            interval=interval,
            dayoffs=dayoffs
        )


class Semester:
    def __init__(self):
        self.number = None
        self.start = None
        self.end = None
        self.lectures: List[ClassData] = []
        self.practical: List[ClassData] = []
        self.labs: List[ClassData] = []

    def parse_dict(self, semester_dict: dict):
        self.number = semester_dict['semester']['number']
        self.start = semester_dict['semester']['start']
        self.end = semester_dict['semester']['end']

        def parse_study_type(name: str, semester_dict: dict) -> List:
            parsed_study_type = []
            for subject_info in semester_dict['classes']['labs']:
                subject = subject_info['subject']
                for day in subject_info['schedule']:
                    for class_data in subject_info['schedule'][day]:
                        class_data = ClassData.parse(
                            class_data,
                            start=self.start,
                            end=self.end,
                            subject=subject
                        )
                        parsed_study_type.append(class_data)
            return parsed_study_type

        self.lectures = parse_study_type('lectures', semester_dict)
        self.parctical = parse_study_type('parctical', semester_dict)
        self.labs = parse_study_type('labs', semester_dict)



import semester_planner
from semester_planner import Semester, SemesterPlanner
from semester_planner import utils
import functools
import json
import argparse
from .utils import parse_args
import os



def cli():
    args = parse_args()
    config = utils.SemesterPlannerConfig()
    semester_data = json.load(open(args.semester))
    semester = Semester()
    semester.parse_dict(semester_data)
    semester_planner = SemesterPlanner(semester)

    if args.command == 'todoist':
        api_token = args.token if args.token else config['TODOIST']['token']
        root_project = args.root_project if args.root_project else config[
            'TODOIST']['root_project']
        something_toggled = args.labs or args.lectures or args.practical

        setup_todoist = functools.partial(
            semester_planner.setup_todoist,
            api_token=api_token,
            root_project=root_project,
            override=args.override
        )

        if not something_toggled or args.all:
            setup_todoist()
        else:
            setup_todoist(
                labs_project="Labs" if args.labs else None,
                lectures_project="Lectures" if args.lectures else None,
                practical_project="Practical" if args.practical else None
            )
    elif args.command == 'config':
        if args.list:
            print(config.to_string(args.config_section.upper()))
        elif args.config_section == 'todoist':
            if args.token:
                config['TODOIST']['token'] = args.token
            if args.root:
                config['TODOIST']['root_project'] = args.root
        config.save()



import todoist
import todoist_colors
from todoist.api import TodoistAPI
import argparse
import json
import os
import sys
import datetime
from via_logger import BeautifulLogger
from via_logger.utils import generate_log_filename
from typing import Union, List
from .data import Class, ClassData, ClassIterator, Semester
from .utils import SemesterPlannerConfig

LOGS_LOCATION = '.'
if not os.path.exists(LOGS_LOCATION):
    os.makedirs(LOGS_LOCATION)


class SemesterPlanner:
    # I don't know whether it works
    # for Windows. I mean /tmp/... It
    # does not exist on Windows, right?
    # So... We'll see what gonna happen;)
    logger = BeautifulLogger.get_instance(
        os.path.join(
            LOGS_LOCATION,
            'semester_planner.log'
        )
    )

    def __init__(self, semester: Semester):
        self.semester = semester

    def setup_todoist(self,
                      api_token: str,
                      root_project: str = "University",
                      labs_project: str = "Labs",
                      lectures_project: str = "Lectures",
                      practical_project: str = "Practical",
                      override: bool = None):
        api = TodoistAPI(api_token)
        api.sync()

        # Get root project from TodoistAPI
        projects: List[todoist.models.Project] = api.state['projects']

        override = False if override is None else override
        for project in projects:
            if project['name'] == root_project:
                if override is not None:
                    root_project = project
                    break
                answer = None
                while answer != 'y' and answer != 'n':
                    answer = input(
                        'Do you want to override existing projects? [y/n] '.format(root_project))
                if answer == 'y':
                    override = True
                root_project = project
                break
        else:
            root_project = api.projects.add(
                name=root_project,
                color=todoist_colors.PURPLE
            )
        api.commit()

        if labs_project is not None:
            labs_project = self.__todoist_create_project(
                api=api,
                project_name=labs_project,
                override=override,
                parent_project=root_project,
                existed_projects=projects,
                color=todoist_colors.BLUE)
            self.__todoist_upload_labs(api, labs_project)

        if lectures_project is not None:
            lectures_project = self.__todoist_create_project(
                api=api,
                project_name=lectures_project,
                override=override,
                parent_project=root_project,
                existed_projects=projects,
                color=todoist_colors.GREEN
            )
            self.__todoist_upload_lectures(api, lectures_project)

        if practical_project is not None:
            practical_project = self.__todoist_create_project(
                api=api,
                project_name=practical_project,
                override=override,
                parent_project=root_project,
                existed_projects=projects,
                color=todoist_colors.ORANGE
            )
            self.__todoist_upload_practical(api, practical_project)

        api.commit()

    def __todoist_create_project(self,
                                 api: todoist.TodoistAPI,
                                 project_name: str,
                                 override: bool,
                                 color: int,
                                 parent_project: todoist.models.Project = None,
                                 existed_projects: List[todoist.models.Project] = None):
        existed_projects = existed_projects if existed_projects is not None else []
        for project in existed_projects:
            if project['name'] == project_name:
                if parent_project and project['parent_id'] != parent_project['id']:
                    continue
                if override:
                    project.delete()
                    break
                return project
        project = api.projects.add(
            name=project_name,
            parent_id=parent_project['id'],
            color=color
        )
        api.commit()
        return project

    def __todoist_upload_labs(self, api: todoist.TodoistAPI, labs_project: todoist.models.Project):
        semester_task_content = '**Labs. Semester #{}**'.format(
            self.semester.number,
            date_string=str(self.semester.end),
            project_id=labs_project['id']
        )
        semester_task = api.items.add(
            semester_task_content,
            project_id=labs_project['id']
        )
        subject = None
        subject_task = None
        subject_lab_count = 0
        for class_data in self.semester.labs:
            if class_data.subject != subject:
                subject = class_data.subject
                subject_task = api.items.add(
                    '{}:'.format(class_data.subject),
                    project_id=labs_project['id'],
                    parent_id=semester_task['id']
                )
                subject_lab_count = 0
            for class_instance in class_data:
                subject_lab_count += 1
                class_instance_content = '{}. Lab #{}'.format(
                    subject,
                    subject_lab_count)

                # Create Todoist-compatible
                # due date string
                class_instance_due_date = class_instance.date + class_data.interval
                if class_instance_due_date < datetime.date.today():
                    class_instance_due_date = datetime.date.today()
                class_instance_due_date = str(class_instance_due_date)

                lab_task = api.items.add(
                    class_instance_content,
                    project_id=labs_project['id'],
                    parent_id=subject_task['id'],
                    date_string=class_instance_due_date
                )

                api.items.add(
                    'Execute the lab (report)',
                    project_id=labs_project['id'],
                    parent_id=lab_task['id']
                )
                api.items.add(
                    'Approve the lab',
                    project_id=labs_project['id'],
                    parent_id=lab_task['id']
                )
            api.commit()

    def __todoist_upload_lectures(self, api: todoist.TodoistAPI, lectures_project: todoist.models.Project):
        semester_task_content = 'Lecture. Semester #{}'.format(
            self.semester.number,
            date_string=str(self.semester.end),
            project_id=lectures_project['id']
        )
        semester_task = api.items.add(
            semester_task_content,
            project_id=lectures_project['id']
        )
        subject = None
        subject_task = None
        subject_lectures_count = 0
        for class_data in self.semester.labs:
            if class_data.subject != subject:
                subject = class_data.subject
                subject_task = api.items.add(
                    '{}:'.format(class_data.subject),
                    project_id=lectures_project['id'],
                    parent_id=semester_task['id']
                )
                subject_lectures_count = 0
            for class_instance in class_data:
                subject_lectures_count += 1
                class_instance_content = '{}. Lecture #{}'.format(
                    subject,
                    subject_lectures_count)

                # Create Todoist-compatible
                # due date string
                class_instance_due_date = class_instance.date + class_data.interval
                if class_instance_due_date < datetime.date.today():
                    class_instance_due_date = datetime.date.today()
                class_instance_due_date = str(class_instance_due_date)

                api.items.add(
                    class_instance_content,
                    project_id=lectures_project['id'],
                    parent_id=subject_task['id'],
                    date_string=class_instance_due_date
                )
            api.commit()

    def __todoist_upload_practical(self, api: todoist.TodoistAPI, practical_project: todoist.models.Project):
        semester_task_content = 'Practical. Semester #{}'.format(
            self.semester.number,
            date_string=str(self.semester.end),
            project_id=practical_project['id']
        )
        semester_task = api.items.add(
            semester_task_content,
            project_id=practical_project['id']
        )
        subject = None
        subject_task = None
        subject_practicals_count = 0
        for class_data in self.semester.labs:
            if class_data.subject != subject:
                subject = class_data.subject
                subject_task = api.items.add(
                    '{}:'.format(class_data.subject),
                    project_id=practical_project['id'],
                    parent_id=semester_task['id']
                )
                subject_practicals_count = 0
            for class_instance in class_data:
                subject_practicals_count += 1
                class_instance_content = '{}. Practical #{}'.format(
                    subject,
                    subject_practicals_count)

                # Create Todoist-compatible
                # due date string
                class_instance_due_date = class_instance.date + class_data.interval
                if class_instance_due_date < datetime.date.today():
                    class_instance_due_date = datetime.date.today()
                class_instance_due_date = str(class_instance_due_date)

                api.items.add(
                    class_instance_content,
                    project_id=practical_project['id'],
                    parent_id=subject_task['id'],
                    date_string=class_instance_due_date
                )
            api.commit()
